! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
 !
! Module: rgsspecplot_mod
!
! Authors:  Jon Rainger - MSSL, Natalie Webb - CESR
!          
! Description: Produces annotated display of RGS spectra
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
!
! Ammendemnts:
! 20th June 2000 - Check added for the number of spectrum files-
!                  if 0 or > 2 files error message displayed and
!                  program terminated.
!                  Code tidied up.
!                  Spectrum order now obtained from parameter
!                  "DSREF2" from SPECTRUM header in spectrum files.
!                  Wavelength scale error corrected.
!                  Code tidied up.
! 15th October 2000 - Program modified to run with output from 
!                     rgsspectrum                  
! 5 April 2001     -  TCRVL1 replaces BETA_REF and TCDLT1 BETA_WID                  
! 9th May 2001    - Ammended code to read latest source file produced 
!                   by rgsources        
!******************************************************************  
module rgsspecplot_mod

  use rgslib
  use saslib
  use dal
  use cal
  use oal
  use param
  use metatask_mod
  use errorHandling
!  use plot_region_mod
  use pointing_mod

  implicit none

  private

  public rgsspecplot
  type(TaskInfo) :: info ! type from metatask_mod
  type VIEWPORT
      real xLeft
      real xRight
      real yBottom
      real yTop
   end type VIEWPORT
contains
  
  !---------------------------------------------------------------------
  subroutine rgsspecplot
  !---------------------------------------------------------------------
    
    real(kind=single)            :: betaRef
    real(kind=single)            :: betaWid
    real(kind=single)            :: tcrvl
    real(kind=single)            :: tcdlt


    real(kind=single)            :: betaRange
    real(kind=single)            :: betaMin
    real(kind=single)            :: offaxis(2)
    integer(kind=int32)          :: order

    real(kind=single)            :: betaCor(1)
    real(single)                 :: wavelength(1)
 !   real(single)                 :: beta(1)

    integer                      :: instrument, largest
 
    character(len=255) :: sourceListSet                               !
    integer(kind=int32) :: sourceID                                    !        
    character(len=1024)          :: plotFile                          !  output filename
    logical(kind=Bool)           :: group, rebin                           ! Group
    character(len=1024)          :: device, cdevice                   ! ploting device e.g /xw
    type(DataSetT)               :: srcListSetH                       ! pointer to source list
    type(DataSetT)               :: inDataSetH                     ! pointer to event list
    type(tableT)                 :: inTableH                          ! pointer to spectrum extension
    type(ColumnT), dimension(3)  :: inColumnH                         ! Input column handle.
    type(ColumnT)                :: statErrColumn

    integer(kind=int32), dimension(:), pointer :: countsP             !
    real(kind=single), dimension(:), pointer :: countsErrorP, ratesP     !
     

    integer(kind=int16), dimension(:), pointer :: channelP            ! 
    integer(kind=int16), dimension(:), pointer :: groupingP           ! 
    
    real, allocatable            :: channel(:), newchannel(:),energy(:)
    integer, allocatable         :: summedCounts(:)
    integer, allocatable         :: summedChannels(:)
    integer, allocatable         :: nChannels(:)
    real, allocatable            :: counts(:), ratetocounts(:),ratetocountserr(:)
    real, allocatable            :: yerror(:), newcounts(:), newyerror(:)

    integer                      :: pgbeg, min_counts, numofbins, howfarthrough                 !
    integer                      :: i, k, bin, nbins, nspectra, looped, min_countother         !

    real                         :: xpts(2), ypts(2)    
    real                         :: labelPos
    real                         :: oldLabelPos, exposuret, runtotal
    character(len=1024)          :: errstring1
    
    character(len=4)             :: energyscale(1 : 15)  
    real                         :: energyLabels(1 : 15)
    character(len=4)             :: wavelengthScale(1 : 7)  
    real                         :: wavelengthLabels(1 : 7)
    real                         :: chipBoundaries(1 : 8)
    real                         :: wavelengthSubTick
    real                         :: energySubTick

    character(len=40)            :: obs_id, exp_idstr, SrcLabel                         !
    character(len=40)            :: exp_id
    character(len=50)            :: spectrumtype, axisunits                        !
    character(len=40)            :: telescope                         !
!    character(len=40)            :: instrume, object, ra, dec, exposure     
    character(len=40)            :: instrume, object, exposure     
    character(len=8)            :: ra, dec                     ! observation details 
    character(len=40)            :: start_obs                         !
    character(len=40)            :: end_obs                           !

    integer(kind=int32)          :: specPix, specDelt, detChans                 !
    real(kind=single)            :: specVal,  originalChannelRange, originalMinChannel 
    integer                      ::noSpectrumSets  
    real(kind = single)          :: xStart
    real(kind=single)            :: minCounts
    real(kind=single)            :: maxCounts, maxError 
    type(VIEWPORT)               :: graphPosition
    logical                      :: drawLine = .FALSE.
    real                         :: xpos, ypos
    real                         :: xMin, xMax, xMinEnergy, xMaxEnergy
    
    energyScale=(/"0.5 ","1.0 ","1.5 ","2.0 ","2.5 ","3.0 ","3.5 ","4.0 ","4.5 ","5.0 ","5.5 ","6.0 ","6.5 ","7.0 ","7.5 "/)      
    energyLabels=(/500.0,1000.0,1500.0,2000.0,2500.0,3000.0,3500.0,4000.0, 4500.0,5000.0,5500.0,6000.0,6500.0,7000.0,7500.0/)

    wavelengthScale = (/ "5 ", "10", "15", "20", "25", "30", "35" /)

    wavelengthLabels = (/ 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5 /)

    chipBoundaries = (/ 0.0416, 0.0458, 0.05, 0.0542, 0.0584, 0.0626, 0.0668, 0.071 /)
   
    graphPosition = VIEWPORT(0.1, 0.95, 0.1, 0.5)

    !******************************************
    ! Get task parameters.
    !******************************************    
    group = booleanParameter("group")
    device = trim(stringParameter("device"))
    sourceListSet = stringparameter("sourcelistset")
    sourceID = intParameter("sourceid")
    plotfile = trim(stringParameter("plotfile"))
    rebin = booleanParameter("rebin")
    min_counts = IntParameter("mincounts") 
 
    if (rebin .and. min_counts < 1) then
      write(errString1, *) "No rebinning will take place"
      call warning("Rebin requested to be less than 1", trim(errstring1))
      rebin = .false.
    end if
 
    min_countother = min_counts

    call tolowerc(device)
    
    if (trim(device(1 : 2)) /= "/x") then   ! Hard copy device
        if (trim(plotFile) == "") then
            write(errString1, *) trim(device)," chosen. Value expected for parameter 'plot_file'"
            call fatal("No hardcopy filename", trim(errstring1))
        else
            call CheckClobber(plotFile)
        end if
        cdevice = trim(plotFile) // device 
    else
        cdevice = device
        if (trim(plotFile) .ne. "") then
            write(errString1, *) trim(device)," chosen. No value required for parameter 'plot_file'"
            call warning("Surplus filename", trim(errstring1))
        end if
    end if
   
        
    noSpectrumSets = parameterCount("spectrumsets")
    if (noSpectrumSets == 0 .or. noSpectrumSets > 2) then
        call fatal("Incorrect number of spectrum sets", &
                   "Has to be either 1 or 2")
    end if
   
    !**********************************************
    ! Open the plotting device.
    !**********************************************
    info = getTaskInfo()

    if (pgbeg(0, cdevice, 1, 1) /= 1) &
        call fatal("badDevice","Unrecognized PGPLOT device")
    
    call pgsubp(1, 2)
    call pgscf(2)


    !**********************************************************
    ! Make sure that the source no. exists in the sources file
    !**********************************************************
    srcListSetH = dataSet(sourceListSet, read)
    call GetOffAxesValues(srcListSetH, sourceId, offaxis)
       
    !*****************************************
    ! Plot the spectrum
    !*****************************************
    do nspectra = 1, noSpectrumSets

        min_counts = min_countother

        ! Get a handle on the input data set.
        inDataSetH = dataSet(stringParameter("spectrumsets", nspectra - 1), read)
     
        ! Get observation identification details for plotting    
        obs_id = stringAttribute(inDataSetH,    "OBS_ID")
        exp_id = stringAttribute(inDataSetH,    "EXP_ID")
        telescope = stringAttribute(inDataSetH, "TELESCOP")
        instrume = stringAttribute(inDataSetH,  "INSTRUME")
        start_obs = stringAttribute(inDataSetH, "DATE-OBS")
        end_obs = stringAttribute(inDataSetH,   "DATE-END")
        if (HasAttribute(inDataSetH,"OBJECT") ) then
          object = stringAttribute(inDataSetH,   "OBJECT")
        else
          object = 'Indef'
        endif
         if (HasAttribute(inDataSetH,"RA_OBJ") ) then
          ra = stringAttribute(inDataSetH,   "RA_OBJ")
        else
          ra = 'Indef'
        endif
        if (HasAttribute(inDataSetH,"DEC_OBJ") ) then
          dec = stringAttribute(inDataSetH,   "DEC_OBJ")
        else
          dec = 'Indef'
        endif
        if (HasAttribute(inDataSetH,"EXPIDSTR") ) then
          exp_idstr = stringAttribute(inDataSetH, "EXPIDSTR")
        else
          exp_idstr = 'Indef'
        endif
    
        ! Set State of CAL.
        instrument = instId(stringAttribute(inDataSetH, "INSTRUME"))
        call CAL_setState(instrument=instrument)
     
        call pgpage
        
        ! Get a handle on SPECTRUM extension.
        inTableH = table(inDataSetH, BlockNumber(inDataSetH,'SPECTRUM'))   

    if ( .not. HasAttribute(inTableH,"HDUCLAS2") ) &
      call warning('RGSSPECPLOT','1 Keyword HDUCLAS2 missing in the input file ')
         select case(StringAttribute(inTableH,"HDUCLAS2"))
         case('TOTAL')
           spectrumtype = 'SRC+BKG'
         case('NET')
           spectrumtype = 'NET'
         case default ! BKG
           spectrumtype = 'BACKGROUND'
         end select

          inColumnH(1) = column(inTableH, "CHANNEL", read)
        channelP => int16Data(inColumnH(1))

   if ( .not. HasAttribute(inColumnH(1),"TCUNI") ) &
      call warning('RGSSPECPLOT','1 Keyword TCUNI1 missing in the input file ')
         select case(StringAttribute(inColumnH(1),"TCUNI"))
         case('Angstrom')
           axisunits = 'angstrom'
         case('rad')
           axisunits = 'rad'
         case default ! BKG
           axisunits = 'rad'
         end select


!    if ( .not. HasAttribute(inTableH,"SOURCEID") ) &
!      call warning('RGSSPECPLOT','NoSourceIdKeyword')
!    if (HasAttribute(inTableH,"SOURCEID") ) then
!      sourceid = int16Attribute(inTableH,"SOURCEID")
!    else
!      sourceid = 1
!    endif
 
   call GetSrcLabel(srcListSetH, sourceId, SrcLabel)
   

!    if (StringAttribute(inTableH,"HDUCLAS2") /= 'NET' ) then
!      spectrumtype = 'BACKGROUND'
!    else 
!      spectrumtype = 'NET'
!    end if
                
        exposure = stringAttribute(inTableH, "EXPOSURE")

    if (nspectra == 1) then
        call PlotExposureDetails(telescope, instrume, &
                 exp_id, start_obs, end_obs, object, exposure, ra, dec, &
                 graphPosition, obs_id, exp_idstr) 
    end if
   

        !************************************************
        ! Get the attribute "RFLORDER" - the spectrum order
        !************************************************         
        call GetInteger32Attribute(order, inTableH, "RFLORDER")
        call GetReal32Attribute(exposuret, inTableH, "EXPOSURE")
  
         !************************************************
        ! Get the attribute "SPECDELT" - if it is not 
        ! present the default value is 1
        !************************************************ 
        call GetInteger32Attribute(specDelt, inTableH,  "SPECDELT", 1)

        !************************************************
        ! Get the attribute "SPECPIX" - if it is not 
        ! present the default value is 0
        !************************************************ 
        call GetInteger32Attribute(specPix, inTableH,  "SPECPIX", 0)

        !************************************************
        ! Get the attribute "SPECVAL" - if it is not 
        ! present the default value is 0
        !************************************************ 
        call GetReal32Attribute(specVal, inTableH,  "SPECVAL", 0.0)


        call GetInteger32Attribute(detChans, inTableH,  "DETCHANS")

        ! Get a handle on the columns.
        inColumnH(1) = column(inTableH, "CHANNEL", read)
        channelP => int16Data(inColumnH(1))

        tcrvl = real32attribute(inColumnH(1), "TCRVL")
        betaRef = tcrvl           
        tcdlt = real32attribute(inColumnH(1), "TCDLT")
        betaWid = tcdlt
 
        
        incolumnH(2) = column(inTableH, 1, read)
        incolumnH(3) = column(inTableH, 3, read)
        nbins = numberOfRows(inTableH)
        if (trim(name(incolumnH(2))) == 'COUNTS') then
          countsP => int32Data(incolumnH(2))
          if (trim(name(incolumnH(3))) == 'STAT_ERR') then
            statErrColumn = column(inTableH, "STAT_ERR", read)
            countsErrorP => real32Data(statErrColumn)
          endif   
        else if(trim(name(incolumnH(2))) == 'RATE') then
          allocate(ratetocounts(nbins),ratetocountserr(nbins))
          ratesP => real32Data(incolumnH(2))
          if (trim(name(incolumnH(3))) == 'STAT_ERR') then
            statErrColumn = column(inTableH, "STAT_ERR", read)
            countsErrorP => real32Data(statErrColumn)
          else    
            countsErrorP => real32Data(incolumnH(2))
          endif
          do i=1,nbins
            ratetocounts(i) = ratesP(i-1)
            ratetocounts(i) = ratetocounts(i)*exposuret
            if (trim(name(incolumnH(3))) == 'STAT_ERR') then
              ratetocountserr(i) = countsErrorP(i-1)
            else 
              ratetocountserr(i) = countsErrorP(i-1)
              ratetocountserr(i) = sqrt(abs(ratetocountserr(i)*exposuret))
            endif   
          end do
        endif

!        statErrColumn = column(inTableH, 1, read)
!        countsErrorP => real32Data(statErrColumn)


        if (group) then   ! Group up channels

            inColumnH(3) = column(inTableH, "GROUPING", read)
            groupingP => int16Data(inColumnH(3))

            call release(inColumnH(3))

            allocate(summedCounts(1 : nbins))
            allocate(summedChannels(1 : nbins))
            allocate(nChannels(1 : nbins))       
       
            bin = 0 
            summedCounts = 0
            summedChannels = 0       
            nChannels = 0

            do i = 1, nbins
                if (groupingP(i - 1) == 1) then
                    bin = bin + 1
                end if
                if (trim(name(incolumnH(2))) == 'COUNTS') then
                  summedCounts(bin) = summedCounts(bin) + real(countsP(i - 1))
                else if(trim(name(incolumnH(2))) == 'RATE') then
                  summedCounts(bin) = summedCounts(bin) + ratetocounts(i) 
                endif
                summedChannels(bin) = summedChannels(bin) + channelP(i - 1)
                nChannels(bin) = nChannels(bin) + 1
            end do
        
            allocate(channel(1 : bin))
            allocate(counts(1 : bin))
            allocate(yerror(1 : bin))
          
            do i = 1, bin
                channel(i) = real(summedChannels(i)) / real(nChannels(i))
                counts(i) = real(summedCounts(i))
                yerror(i) = sqrt(counts(i))
            end do
          
            deallocate(summedCounts)
            deallocate(summedChannels)
            deallocate(nChannels)   
         else ! No grouping          

             allocate(channel(1 : nbins), newchannel(1:nbins))
             allocate(counts(1 : nbins), newcounts(1:nbins))
             allocate(yerror(1 : nbins), newyerror(1:nbins))         

             bin = nbins
          
             outerloop: do i = 1, bin-1
               channel(i) = channelP(i - 1)
               if (trim(axisunits) == 'angstrom' .and. .not.(rebin) ) then
                 channel(i) = (BetaRef+BetaWid*channelP(i - 1))/real(order)
               end if
                 countloop: if (trim(name(incolumnH(2))) == 'COUNTS') then
                   getcounts: if (countsP(i - 1) .ge. -100.0) then
                     getcounts1: if (i==1) then
                       counts(i) = countsP(i - 1)
!Use only poissonian errors, if no other errors i.e. statistical, in input files
                       if (trim(name(incolumnH(3))) == 'STAT_ERR') then
                         yerror(i) = max(0.0, countsErrorP(i-1))
                       else 
                         yerror(i) = max(0.0, sqrt(abs(real(countsP(i - 1)))))
                       endif
                     else 
                       counts(i) = countsP(i - 1)
                       if (trim(name(incolumnH(3))) == 'STAT_ERR') then
                         yerror(i) = max(0.0, countsErrorP(i-1))
                       else 
                         yerror(i) = max(0.0, sqrt(abs(real(countsP(i - 1)))))
                       endif
                     end if getcounts1
                   end if getcounts
                     else if(trim(name(incolumnH(2))) == 'RATE') then
                     getcounts2: if (ratetocounts(i) .ge. -100.0) then
                       counts(i) = ratetocounts(i)
                       yerror(i) = max(0.0, ratetocountserr(i))
                     else
                      counts(i) = -0.000001
                       yerror(i) = 0.0
                     end if getcounts2
                   end if countloop
                 end do outerloop         
             end if  ! End of "if group" construct

! Moved this as it was taking the minimum channel as the first channel 
! in the binned spectrum when rebinning was required, so the wavelength 
! scale was wrong (SSC-SPR-3628)

!        if (nspectra == 1) then
          xMin = channel(1)-((channel(2)-channel(1))/2.0)
          largest = maxval(maxloc(channel))
          xMax = channel(largest)+((channel(largest)-channel(largest-1))/2.0)
!        else
!         xMin=xMin
 !         xMax=xMax
 !         xMinEnergy=xMinEnergy
 !         xMaxEnergy=xMaxEnergy
 !       end if


        if (rebin .and. sum(counts) < 5.0*min_counts) then
          min_counts = int(sum(counts)/5.0)
          if (rebin .and. min_counts < 1) then
            write(errString1, *) "No rebinning will take place"
            call warning("Average counts per bin less than 1", trim(errstring1))
            rebin = .false.
!added to the following 6 lines to fix problem with the x-axis display if 
!the program decides that no rebinning should occur 
!CHECK here
            do i = 1, bin-1
               channel(i) = channelP(i - 1)
 !              if (trim(axisunits) == 'angstrom' .and. order == 1) then
!ChangeLog 2.21, this helps with the second order too 
              if (trim(axisunits) == 'angstrom') then
                 channel(i) = (BetaRef+BetaWid*channelP(i - 1))/real(order)
               end if
            end do
          else if (rebin .and. min_counts > 1) then
          write(errString1, *) "Replacing minimum counts per bin by", min_counts
          call warning("Rebinning results in less than 5 bins", trim(errstring1))
          end if
        end if

!Where the data is rebinned if requested
        newchannel=0.0; newcounts=0.0; newyerror=0.0; numofbins=0; k=1
        howfarthrough=1; looped=0
        if (rebin) then
          thekloop: do while (howfarthrough < bin)
            howfarthrough=howfarthrough+looped; numofbins=0; runtotal=0.0
            looped=0
            theiloop: do i=howfarthrough,bin-1
              if (runtotal > real(min_counts)) exit theiloop
              numofbins=numofbins+1
              newchannel(k) = newchannel(k) + channel(i)
              if (yerror(i) > tiny(1.0)) then
                runtotal = runtotal + counts(i)
                newcounts(k) = newcounts(k) + counts(i)
!                newyerror(k) = newyerror(k) + (1.0/(yerror(i)**2.0))
!                newchannel(k) = newchannel(k) + channel(i)
!                numofbins=numofbins+1
              end if
              looped=looped+1
            end do theiloop
            if (k < bin-1) then
              newchannel(k)=newchannel(k)/real(numofbins)
              newcounts(k)=newcounts(k)/real(numofbins)
              newyerror(k)=sqrt(abs(newcounts(k)))
              k=k+1
            end if
          end do thekloop
          deallocate(channel, counts, yerror)
          allocate(channel(k-2), counts(k-2), yerror(k-2), energy(k-2))
          channel=0.0; counts=0.0; yerror=0.0
          do i=1,k-2
            channel(i)=newchannel(i)
            if (trim(axisunits) == 'angstrom') then
              channel(i) = (BetaRef+BetaWid*newchannel(i))/real(order)
            end if
            counts(i)=newcounts(i)
            yerror(i)=newyerror(i)
          end do
          nbins=k-2
          bin=k-2
        end if 
        minCounts = 1e10
        maxCounts = -1e10
        maxError = -1e10

!            if (trim(axisunits) == 'angstrom') then
                !energy=hc/lambda*e eV (/1000 for keV)
!              energy = (6.6260755e-34*2.99792458e8)/(channel*1.60217733e-26)
!            end if

!        if (nspectra == 1) then
          xMin = channel(1)-((channel(2)-channel(1))/2.0)
          xMinEnergy=(6.6260755e-34*2.99792458e8)/(xMin*1.60217733e-26)
          largest = maxval(maxloc(channel))
          xMax = channel(largest)+((channel(largest)-channel(largest-1))/2.0)
          xMaxEnergy=(6.6260755e-34*2.99792458e8)/(xMax*1.60217733e-26)
!        else
!          xMin=xMin
!          xMax=xMax
!          xMaxEnergy=xMaxEnergy
 !       end if

        do i = 1, nbins-1
        if (trim(axisunits) == 'rad') then
            channel(i) = (specVal - specPix * specDelt - 0.5 * &
                          specDelt) + channel(i) * specdelt
         end if
             if (counts(i) .ge. -500.0) then
                minCounts = min(minCounts, counts(i))
                maxCounts = max(maxCounts, counts(i))
                maxError  = max(maxError, yerror(i))
            endif
        end do
        
        if (minCounts > 0.0) minCounts = 0.0
        if (maxCounts < maxError) maxCounts = maxError 

        call pgrnge(minCounts, maxCounts, minCounts, maxCounts)
        ! Plot graph.
        call pgsvp(graphPosition%xLeft, graphPosition%xRight, 0.1, 0.62)
!was here
        if (minCounts .eq. maxCounts) then
            call pgswin(xMin, xMax, 0.9 * minCounts, 1.1111 * (maxCounts + 1.0))
        else
            call pgswin(xMin, xMax, minCounts, maxCounts)
       end if
        call pgsci(1)


        !*******************************************
        ! Plot different colours for region overlaps
        !*******************************************
        call pgsls(1)   ! Added on 09-11-2000
        drawline = .FALSE.
  
        call pgsci(1)
        call pgslw(1)
        call pgmove(xpos, ypos)
        drawline = .TRUE.
        call pgbin(nbins,channel, counts, .TRUE.)
        drawline = .FALSE.
  
        do i = 1, nbins
          call pgsci(15)
          call pgslw(1)
          if (drawLine) then
            call pgsci(15)   
            call pgmove(xpos, ypos)
            if (yerror(i) .ne. 0.0) then
              call pgdraw(channel(i), yerror(i))
            end if
          endif
          drawline = .TRUE. 
          xpos = channel(i)
          ypos = yerror(i)  
        end do

        drawline = .FALSE.

        call pgsci(1)   
!        call pgerry(nbins , channel , counts+yerror , counts-yerror, 0.0)
!        call pgsci(1)   
              
        !***************************************
        ! Display the maximum-sized error bar
        !***************************************       
        call HeapSort(yerror, bin) 
!        call pgptxt(xmin + (xmax - xmin) / 20.0,  (maxcounts - (minCounts + maxCounts) / 10.0) , &
!                    0.0, 0.0, "-Maximum error")
     
!        call pgerr1(6, xmin + (xmax - xmin) / 25.0, (maxcounts - (minCounts + maxCounts) / 10.0), &
!                yerror(bin), 2.0)
       
        call pgsci(1)
        call pgslw(2)   
        call pgsch(1.2)  ! The tick and character sizes on the axes
        call pgbox('bts', 0.0,0, 'bcts', 0.0, 0)      
        call pgsch(1.0)
        call pgslw(2)  
        call pgbox('n', 0.0,0, 'n', 0.0, 0)      
        call pgsch(1.25)
        call pgslw(4)
        if (trim(axisunits) == 'rad') then
          call pglab('Beta Channel ', 'Counts/channel', '')  ! Changed from counts/s
        else
          call pglab('Wavelength (\A)', 'Counts/channel', '')  ! Changed from counts/s
        end if
        call pgslw(1)

!call pgbox again to plot the energy on the top

!       if (trim(axisunits) == 'angstrom') then  
!          if (minCounts .eq. maxCounts) then
!            call pgswin(xMinEnergy, xMaxEnergy, 0.9 * minCounts, 1.1111 * (maxCounts + 1.0))
!          else
!            call pgswin(xMinEnergy, xMaxEnergy, minCounts, maxCounts)
!          end if
!        call pgbox('cmts2', 0.0,0, 'cts', 0.0, 0)      
!        call pgsch(1.0)
!        call pgslw(2)  
!        call pgbox('m', 0.0,0, 'n', 0.0, 0)      
!        end if
               

        ! Plot additional axes.
        xpts(1) = 0.0   ! REAL(MINVAL(channel))
        ypts(1) = 50.0  ! 
        xpts(2) = 100.0 ! REAL(MAXVAL(channel))
        ypts(2) = 50.0  !
        

        call pgsvp(graphPosition%xLeft, graphPosition%xRight, 0.57, 0.67)
        call pgswin(0.0, 100.0, 0.0, 100.0)
        CALL pgline(2, xpts, ypts)
        

       
       ! Important to pick up evselect user input wrt binning.
        if (rebin) then
!          if (trim(axisunits) == 'angstrom') then
!            originalMinChannel = xMin; originalChannelRange = 3399.5
!          else
            originalMinChannel = xMin; originalChannelRange = xMax
!          end if
        else
          originalMinChannel = specVal - specPix * specDelt - 0.5 * specDelt
          originalChannelRange = specDelt * detChans
        end if 

        betaMin = BetaRef + 0.5 * betaWid + originalMinChannel * BetaWid 
        betaRange = originalChannelRange * BetaWid

        call pgsch(1.0)
        if (trim(axisunits) == 'rad') then
        ! Add Minor tick marks to wavelength  axis.
          do i = 1, 40
              wavelengthSubTick = 0.1 * real(i)
              wavelength(1) = wavelengthSubTick

                betaCor = lambda2Beta(wavelength, -order, REAL(offaxis(1)))
                ! convert corrected beta to position of label on axis.
                labelPos = 1.0 - (betaCor(1)- betaMin) / betaRange

              call pgtick(100.0, 50.0, 0.0, 50.0, labelPos, &
                        0.4, 0.0, 0.7, 180.0, "")
          end do

!          if (trim(axisunits) == 'rad') then
            wavelength(1) = 0.0
            betaCor = lambda2Beta(wavelength, -order, real(offaxis(1)))
          
        !  convert corrected beta to position of label on axis.
            xStart = 1.0 - (betaCor(1) - betaMin) / betaRange
            wavelengthSubTick = 0.1 * 40.0
            wavelength(1) = wavelengthSubTick
            betaCor = lambda2Beta(wavelength, -order, REAL(offaxis(1)))
          
        ! convert corrected beta to position of label on axis.
            labelPos = 1.0 - (betaCor(1) - betaMin) / betaRange
 !       end if    
        
        ! Add major tick Marks and labels for 7 pre-determined wavelength values.
          do i = 1, 7 
            wavelength(1) = wavelengthlabels(i)
            betaCor = lambda2Beta(wavelength, -order, real(offaxis(1)))
            ! convert corrected beta to position of label on axis.
            labelPos = 1.0 - (betaCor(1) - betaMin) / betaRange
              if (labelPos >= 0.0 .and. labelPos <= 1.0) then
                  call pgtick(100.0, 50.0, 0.0, 50.0, labelPos,&
                          0.8, 0.0, 0.7, 180.0, wavelengthScale(i))
              end if    
          end do
          call pgslw(3)
          call pgptxt(50.0, 100.0, 0.0, 0.5, "Wavelength (\A)")
          call pgslw(1)
          call pgsvp(graphPosition%xLeft, graphPosition%xRight, 0.66, 0.76)
          call pgswin(0.0, 100.0, 0.0, 100.0)
          call pgslw(2)
          call pgline(2, xpts, ypts)
        end if

        ! Add Minor tick marks to energy axis.
        do i = 1, 85
         ! Using the radian values for TCRVL1 and TCDLT1 if the spectrum is in
         ! angstroms
!            if (trim(axisunits) == 'angstrom') then
!               BetaRef = 3.57852391898632e-2
!               BetaWid = 1.20799995784182e-5
!               detChans = 3399.5
!            endif
            energySubTick = 0.1 * real(i) * 1000.0
            wavelength(1) = 1239.8_double / energySubTick
            betaCor = lambda2Beta(wavelength, -order, offaxis(1))
            
            ! convert corrected beta to position of label on axis.
            labelPos = 1.0 - (betaCor(1) - betaMin) / betaRange
            call pgtick(100.0, ypts(1), 0.0, ypts(1), labelPos,&
                        0.4, 0.0, 0.7, 180.0, "")
        end do       
        call pgslw(1)
        oldLabelPos = 100.0
        do i = 1, 15
            wavelength(1) = 1239.8_double / energylabels(i)
            betaCor = lambda2Beta(wavelength, -order, offaxis(1))             
            ! convert corrected beta to position of label on axis.
            labelPos = 1.0 - (betaCor(1) - betaMin) / betaRange 
           if (labelPos >= 0.0 .and. labelPos <= 1.0) then
                ! The following is to prevent overlapping of labels.
                if (abs(labelPos - oldLabelPos) > 0.05) then    
                    call pgtick(100.0, ypts(1), 0.0, ypts(1), labelPos,&
                           0.8, 0.0, 0.7, 180.0, energyScale(i)) 
                    oldLabelPos = labelPos  
                else
                    call pgtick(100.0, ypts(1), 0.0, ypts(1), labelPos,&
                          0.8, 0.0, 0.7, 180.0, "")
                end if   
           end if   
   

!            call release(inColumnH(1))
!            call release(incolumnH(2))
!            call release(inTableH)
!            call release(inDataSetH)
            if (allocated(channel)) deallocate(channel)
            if (allocated(energy)) deallocate(energy)
            if (allocated(counts)) deallocate(counts)
            if (allocated(yerror)) deallocate(yerror)
            if (allocated(newchannel)) deallocate(newchannel)
            if (allocated(newcounts)) deallocate(newcounts)
            if (allocated(newyerror)) deallocate(newyerror)
            if (allocated(ratetocounts)) deallocate(ratetocounts)
            if (allocated(ratetocountserr)) deallocate(ratetocountserr)
        end do

        call pgslw(3)
        if (trim(axisunits) == 'rad') then
          call pgptxt(50.0, 105.0, 0.0, 0.5, "Energy (keV)")
        end if
        call pgslw(1)
        call Annotate(sourceID, instrume, order, spectrumtype, graphPosition, min_counts, rebin, SrcLabel) 
        call release(inColumnH(1))
        call release(incolumnH(2))
!        call release(statErrColumn)
        call release(inTableH)
        call release(inDataSetH)
    
    end do   

    call pgmtxt("b", 14.5, -0.12, 0.0, trim(nameOf(info))//' v'//trim(versionOf(info)))
    call release(srcListSetH)       


    call pgend

  end subroutine rgsspecplot

!---------------------------
 SUBROUTINE TOLOWERC(STRING)
!---------------------------

! Sets a string to lower case

   CHARACTER(*),INTENT(INOUT) :: STRING
   INTEGER                    :: I
   INTEGER                    :: LENSTR, NEWCH
   CHARACTER(1)               :: CHA

   LENSTR = LEN(STRING)
   DO I = 1,LENSTR
      CHA = STRING(I:I)
      IF ( LGE(CHA,'A') .AND. LLE(CHA,'Z') ) THEN
         NEWCH = ICHAR(CHA)+ICHAR('a')-ICHAR('A')
         STRING(I:I) = CHAR(NEWCH)
      END IF
   END DO

 END SUBROUTINE TOLOWERC
!******************************************************
! subroutine Annotate
! Adds the source id and spectrum order to the top 
! of the graph.
!*****************************************************
subroutine Annotate(sourceID, instrument, order, spectrumtype, graphPosition, min_counts, rebin, SrcLabel)
    integer(kind=int32), intent(in) :: sourceID, order
    integer, intent(in) :: min_counts
    character(len=*), intent(in) :: instrument
    type(VIEWPORT), intent(in) ::  graphPosition
    logical(kind=Bool)         :: rebin 
    character(len=40)            :: SrcLabel     
     character(len=50) :: text, spectrumtype  
    character(len=80) :: text1
    character(len=50) :: format(4) = (/'(a12, i1, a20, i1)', & 
                                       '(a12, i2, a20, i1)',  &
                                       '(a12, i3, a20, i1)',  &
                                       '(a12, i4, a20, i1)'/)
    
    write(text, format(int(log10(real(sourceID) + 0.0001)) + 1)) &
                "SOURCE ID: ", sourceID, "- SPECTRUM ORDER: ", order


    if (rebin) then
      write(text1,*) " ", trim(spectrumtype), " SPECTRUM, Min. cts/bin = ", min_counts  
    else
     write(text1,*) " ", trim(spectrumtype), " SPECTRUM, No rebinning"
    end if 
   
    call pgsvp(graphPosition%xLeft, graphPosition%xRight, 0.45, 0.90)
    call pgswin(0.0, 1.0, 0.0, 1.0)

    call pgsch(1.2)
    call pgsls(2)
    call pgslw(3)   
    call pgmtext('T', -2.0, -0.16, -0.16, trim(text))
    call pgmtext('T', -3.5, -0.0, 0.5, trim(SrcLabel))
    call pgmtext('T', -2.0, 1.0, 1.0, trim(text1))
    call pgslw(1)   
end subroutine Annotate

!******************************************************
! PlotExposureDetails
! Add the exposure details to the plot.
!*****************************************************
subroutine PlotExposureDetails(telescope, instrument, &
   exposureID, startObs, endObs, object, exposure, ra, dec, graphPosition, &
          obs_id, exp_idstr)
   
    character(len=*), intent(in) ::  telescope
    character(len=*), intent(in) ::  instrument
    character(len=*), intent(in) ::  startObs
    character(len=*), intent(in) ::  endObs, object, exposure, ra, dec
    character(len=*), intent(in) ::  exposureID, obs_id, exp_idstr
    character(len=80)            :: text     
    type(VIEWPORT), intent(in) ::  graphPosition
 
    call pgsvp(graphPosition%xLeft, graphPosition%xRight, 0.45, 0.90)
    call pgswin(0.0, 1.0, 0.0, 1.0)
    call pgsch(0.9)
    call pgslw(3)
    text = trim(telescope)//" - "//trim(instrument)//" - OBJECT: "//trim(object)//" - RA: "//trim(ra)//" - DEC: "//trim(dec) 
    call pgmtxt('T', 2.2, -0.16, -0.16, text)
 
    text = "OBS-ID: " // trim(obs_id) // " - EXP-ID: " // trim(exp_idstr) // " - Exp. Time: " // trim(exposure) 
    call pgmtxt('T', 1.0, -0.17, -0.17, text)

    text = "Key:  - data"  
    call pgmtxt('T', -0.2, -0.08, -0.08, text)

    call pgsci(15)
    text = "- errors"  
    call pgmtxt('T', -0.2, 0.1, 0.1, text)
    call pgsci(1)

    text = "DATE-OBS " // trim(startObs)
    call pgmtxt('T', 2.2, 1.0, 1.0, TRIM(text))

    text = "DATE-END " // trim(endObs) 
    call pgmtxt('T', 1.0, 1.0, 1.0, trim(text))

    call pgslw(1)   

end subroutine PlotExposureDetails

!*************************************************
! CreateExtName
! Forms a string label for the extension name.
!*************************************************
subroutine CreateExtName(no, instrument, extName)   
    integer (kind=int8), intent(in) :: no
    character (len=*), intent(in) :: instrument
    character (len=*), intent(inout) :: extName
 
    if (no < 10) then
        write(extName, '(a4, a4, i1, a8)') TRIM(instrument), &
              "_SRC", no, "_SPATIAL"
    else
       write(extName, '(a4, a4, i2, a8)') TRIM(instrument), &
              "_SRC",  no, "_SPATIAL"
    end if

end subroutine CreateExtName   

!***********************************
! CheckClobber
!   If SAS_CLOBBER set then checks 
!   that filename doesn't already exist. 
!**********************************
subroutine CheckClobber(fileName)
    character(len=*), intent(in) :: fileName

    if (.not. clobber()) then
        open(unit = 4, FILE = fileName, STATUS = 'OLD', ERR = 1)
        close(unit = 4)
        call fatal("Plot file already exists", "SAS-CLOBBER set")
    1   return
    end if
end subroutine CheckClobber
!*******************************************************
! GetInteger32Attribute
! Attempts to read an integer attribute from a fits file.
! Returns TRUE if attribute exits- FALSE otherwise.
!******************************************************
subroutine GetInteger32Attribute(value, table, attribute, &
    defaultValue)
    integer(kind=int32), intent(out) :: value
    type(TableT), intent(in) :: table
    character(len=*), intent(in) :: attribute
    integer(kind=int32), intent(in), optional :: defaultValue
       
    if (hasAttribute(table, attribute)) then
        value = int32Attribute(table, attribute)
    else
        if (present(defaultValue)) then
            value = defaultValue
        else
            call fatal("Attribute could not be found!", attribute)
        end if
    end if
end subroutine GetInteger32Attribute 

!*******************************************************
! GetReal32Attribute
! Attempts to read a real32 attribute from a fits file.
! Returns TRUE if attribute exits- FALSE otherwise.
!******************************************************
subroutine GetReal32Attribute(value, table, attribute, &
    defaultValue)
    real(kind=single), intent(out) :: value
    type(TableT), intent(in) :: table
    character(len=*), intent(in) :: attribute
    real(kind=single), intent(in), optional :: defaultValue
       
    if (hasAttribute(table, attribute)) then
        value = real32Attribute(table, attribute)
      else
  
          if (present(defaultValue)) then
              value = defaultValue
          else
              call fatal("Attribute could not be found!", attribute)
          end if
    end if
end subroutine GetReal32Attribute  
 
!********************************************************
! subroutine GetOffAxesValues 
!
!
!********************************************************
subroutine GetOffAxesValues(srcListSetH, sourceId, offAxis)
    type(DataSetT), intent(inout) :: srcListSetH
    integer(kind=int32), intent(in) :: sourceID
    real(kind=single), intent(inout) :: offAxis(2)
    type(TableT) :: srcTableH      
    type(ColumnT) :: data
    real(kind=single), pointer, dimension(:) :: dataPointer
    real(kind=double) :: degreesToRadians = 3.141592654d0 / 180.0d0 
    
    srcTableH = table(srcListSetH, "SRCLIST")    ! 9th May 2001 changed "SOURCES" to "SRCLIST" for new version of rgssources           
    !*********************************************************
    ! Need to get RGS_DISP & RGS_XDSP from the "SOURCES" 
    ! table - using the values from the row for the source number
    !*********************************************************
    data = column(srcTableH, "DELTA_DISP", read)        ! 9th May 2001 changed "RGS_DISP" to "DELTA_DISP"    
    if (sourceID .gt.  numberOfRows(srcTableH)) then
        call fatal("No table entry for sourceID", "sourceID doesn't exist in sources file")
    end if


    dataPointer => real32Data(data)
    offAxis(1) = dataPointer(sourceId - 1)
           
    data = column(srcTableH, "DELTA_XDSP", read)   ! 9th May 2001 changed "RGS_XDSP" to "DELTA_XDSP"    
    dataPointer => real32Data(data)
    offAxis(2) = dataPointer(sourceId - 1)
 
    call release(data)
    call release(srcTableH)
      
    !******************************************
    ! Convert values from arcminutes to radians
    !******************************************
    

       offAxis(1) = offAxis(1) * degreesToRadians / 60.0d0 
       offAxis(2) = offAxis(2) * degreesToRadians / 60.0d0

end Subroutine GetOffAxesValues

!********************************************************
! subroutine GetSrcLabel 
!
!
!********************************************************
subroutine GetSrcLabel(srcListSetH, sourceId, SrcLabel)
    type(DataSetT), intent(inout) :: srcListSetH
    integer(kind=int32), intent(in) :: sourceID
    type(TableT) :: srcTableH      
    type(ColumnT) :: dat
    real(kind=single), pointer, dimension(:) :: dataPointer
    character(len=40) :: SrcLabel
    
    srcTableH = table(srcListSetH, "SRCLIST")    

    dat = column(srcTableH, "LABEL", read)        

    if (sourceID .gt.  numberOfRows(srcTableH)) then
        call fatal("No table entry for sourceID", "sourceID doesn't exist in sources file")
    end if

    SrcLabel = stringCell(dat, sourceID-1)
 
    call release(dat)
    call release(srcTableH)
      
end Subroutine GetSrcLabel

!************************************************************
! Subroutine HeapSort
! Sorts  one-dimensional array(1.. size) int ascending order.
!************************************************************
subroutine HeapSort(array, size)
    integer(kind = int32), intent(in) :: size
    real(kind = single), intent(inout), dimension(1 : size) :: array
    integer(kind = int32) :: i, ir, j, l
    real(kind = single) :: rra
  
    if (size < 2) return

    l = size / 2 + 1
    ir = size   

    do 
        if (l > 1) then
            l = l - 1
            rra = array(l)    
        else
            rra = array(ir)
            array(ir) = array(1)
            ir = ir - 1
            if (ir <= 1) then
                array(1) = rra
                exit
            end if
        end if

        i = l
        j = l + l
        do while(j <= ir)
       
            if (j < ir) then
                if(array(j) < array(j + 1)) j = j + 1
            end if
            if (rra < array(j)) then
                array(i) = array(j)
                i = j
                j = j + j
            else
                exit
            end if
            
        end do   
        array(i) = rra    
    end do
end subroutine HeapSort


end module rgsspecplot_mod





















