#!/usr/bin/env python
# 
# ESA (C) 2000-2021 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.


import pysas.rgsspecplot.rgsspecplot as rgsspecplot
import numpy as np



def test_collect_fits_info():
    info = rgsspecplot.collect_fits_info('SPECTRUM1010.FIT')
    assert len(info) == 10


def test_get_src_label_and_offaxes():
    src_label, offaxis = rgsspecplot.get_src_label_and_offaxes('rgssources.ds', 1)
    assert src_label == 'PROPOSAL' and len(offaxis) == 2


def test_lambda2beta():
    beta = rgsspecplot.lambda2beta(1, 0, np.array((0.1, 0.2)), -1, 0.2)
    assert round(beta[0], 3) == 0.495 and round(beta[1], 3) == 0.676


def test_general():
    parsed = dict()
    parsed.update({'group' : 'no'})
    parsed.update({'rebin' : 'no'})
    parsed.update({'sourceid' : '1'})
    parsed.update({'format' : 'pdf'})
    parsed.update({'plotfile' : 'plot1'})
    parsed.update({'dark_mode' : 'yes'})
    parsed.update({'spectrumsets' : 'SPECTRUM0101.FIT SPECTRUM0102.FIT'})
    parsed.update({'sourcelistset' : 'rgssources.ds'})
    parsed.update({'n_points' : '200'})
    rgsspecplot.run(parsed)
    assert True
    
    
def test_general_grouping():
    parsed = dict()
    parsed.update({'sourceid' : '1'})
    parsed.update({'n_points' : '200'})
    parsed.update({'group' : 'yes'})
    parsed.update({'plotfile' : 'plot2'})
    parsed.update({'rebin' : 'no'})
    parsed.update({'dark_mode' : 'no'})
    parsed.update({'format' : 'png'})
    parsed.update({'spectrumsets' : 'grouped_file.fits'})
    parsed.update({'sourcelistset' : 'group_sources.FIT'})
    rgsspecplot.run(parsed)
    assert True
